// SANOCompare.cpp: implementation of the CSANOCompare class.
// Copyright (c) 2014 by David Wincelberg
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include <cctype>		// for isdigit, tolower, isspace
#include <cstdlib>		// for atoi

#include "SANOCompare.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSANOCompare::CSANOCompare()
{

}


CSANOCompare::~CSANOCompare()
{

}


// Action:	Implements less-than for SANO.
bool CSANOCompare::operator() (const string s1, const string s2) const
{
//	return SimpleANCompare (s1.c_str(), s2.c_str()) < 0;
	return SimpleANCompare (s1, s2) < 0;
}


// Action:	Simple alphanumeric comparisons with C strings.
// Notes:	Works when there is a digit in the same starting place in
//			each filename.	Limited to numbers smaller than 2^31,
//			around 2 billion.
int CSANOCompare::SimpleANCompare (const char *pszName1, const char *pszName2) const
{
	register const char *pszN1 = pszName1, *pszN2 = pszName2;
	int nTest;

	while (*pszN1 != '\0' && *pszN2 != '\0') {
		if (isdigit (*pszN1) && isdigit (*pszN2)) {
			if ((nTest = atoi (pszN1) - atoi (pszN2)) != 0)
				return nTest;

			// Moves past the numbers.
			while (isdigit (*pszN1)) ++pszN1;
			while (isdigit (*pszN2)) ++pszN2;
		}
		else {
			nTest = tolower (*pszN1) - tolower (*pszN2);

			if (nTest != 0)
				return nTest;
			else {
				++pszN1;  ++pszN2;
			}
		}
	}

	return (*pszN1 - *pszN2);	// At least one string has ended.
}	// SimpleANCompare


// Action:	Simple alphanumeric comparisons with std:strings.
// Notes:	Works when there is a digit in the same starting place in
//			each filename.	Limited to numbers smaller than 2^31,
//			around 2 billion.
//			Standard lib function stoi starts at the beginning of a std::string;
//			no offset is permitted.  It may throw exceptions.
int CSANOCompare::SimpleANCompare (string str1, string str2) const
{
	int nTest;
	string::iterator pos1 = str1.begin(), end1 = str1.end();
	string::iterator pos2 = str2.begin(), end2 = str2.end();
	string::iterator after1, after2;

	while (pos1 != end1 && pos2 != end2) {
		if (isdigit (*pos1) && isdigit (*pos2)) {
			if ((nTest = stoi (pos1, end1, after1) - stoi (pos2, end2, after2)) != 0)
				return nTest;

			// Moves past the numbers.
			pos1 = after1;
			pos2 = after2;
		}
		else {
			nTest = tolower (*pos1) - tolower (*pos2);

			if (nTest != 0)
				return nTest;
			else {
				++pos1;  ++pos2;
			}
		}
	}

	// At least one string has ended.
	if (pos1 == end1 && pos2 == end2)
		return 0;
	else if (pos1 == end1)
		return -1;
	else
		return +1;
}	// SimpleANCompare


// Action:	Calculates the numeric value of part of a string.
int CSANOCompare::stoi (string::iterator pos, string::iterator end,
						string::iterator &after) const
{
	int nSum = 0;
	bool bNegative = false;

	// Skips whitespace.
	while (pos != end && isspace (*pos))
		++pos;

	if (pos == end) {
		after = end;
		return 0;
	}

	// Looks for a sign character.
	if (*pos == '-') {
		bNegative = true;
		++pos;
	}
	else if (*pos == '+')
		++pos;

	if (pos == end) {
		after = end;
		return 0;
	}

	// Accumulates digits.
	while (pos != end && isdigit (*pos)) {
		nSum = 10 * nSum + (*pos - '0');
		++pos;
	}

	after = pos;

	if (bNegative)
		return -nSum;
	else
		return nSum;
}	// stoi
